@@ -7,22 +7,23 @@
 # endif
 
 cap_value_t cap_keep_startup[] = {
+# if defined(USE_PAM) && defined(CAP_AUDIT_WRITE)
+    CAP_AUDIT_WRITE,
+# endif
     CAP_SETGID,
     CAP_SETUID,
     CAP_CHOWN,
     CAP_NET_BIND_SERVICE,
     CAP_SYS_CHROOT,
     CAP_SYS_NICE,
-    CAP_NET_ADMIN,
     CAP_DAC_READ_SEARCH
 };
 
 cap_value_t cap_keep_login[] = {
 # ifdef WITHOUT_PRIVSEP
     CAP_SETUID,
-    CAP_NET_BIND_SERVICE,
+    CAP_NET_BIND_SERVICE
 # endif
-    CAP_NET_ADMIN
 };
 
 #endif
