@@ -105,9 +105,6 @@
 #ifdef HAVE_UTIME_H
 # include <utime.h>
 #endif
-#ifdef WITH_RFC2640
-# include <iconv.h>
-#endif
 
 #include "mysnprintf.h"
 
@@ -362,13 +359,13 @@ void dornto(char *name);
 void dostou(void);
 void dofeat(void);
 void domlst(const char * const file);
+void domlsd(const char * const base);
 void dositetime(void);
 int ul_check_free_space(const char *name, const double min_space);
-void mappedtov4(struct sockaddr_storage *ss);
 void disablesignals(void);
 void getnames(void);
-void donlist(char *arg, const int on_ctrlconn, const int opt_l_,
-             const int opt_a_, const int split_args);
+void dolist(char *arg, const int on_ctrlconn);
+void donlst(const char *base);
 void opendata(void);
 void closedata(void);
 void client_fflush(void);
@@ -398,9 +395,6 @@ unsigned int zrand(void);
 void simplify(char *subdir);
 int checkprintable(register const char *s);
 void delete_atomic_file(void);
-#ifdef WITH_RFC2640
-char *charset_fs2client(const char * const string);
-#endif
 void usleep2(const unsigned long microsec);
 int mysnprintf(char *str, size_t size, const char *format, ...);
 
@@ -421,6 +415,9 @@ extern int opt_a, opt_C, opt_d, opt_F, opt_l, opt_R;
 # ifndef TLS_CERTIFICATE_FILE
 #  define TLS_CERTIFICATE_FILE TLS_CONFDIR "/pure-ftpd.pem"
 # endif
+# ifndef TLS_KEY_FILE
+#  define TLS_KEY_FILE TLS_CERTIFICATE_FILE
+# endif
 # ifndef TLS_DHPARAMS_FILE
 #  define TLS_DHPARAMS_FILE TLS_CONFDIR "/pure-ftpd-dhparams.pem"
 # endif
@@ -461,6 +458,14 @@ extern int opt_a, opt_C, opt_d, opt_F, opt_l, opt_R;
 # endif
 #endif
 
+#ifndef CERTD_PID_FILE
+# ifdef NON_ROOT_FTP
+#  define CERTD_PID_FILE CONFDIR "/pure-certd.pid"
+# else
+#  define CERTD_PID_FILE STATEDIR "/run/pure-certd.pid"
+# endif
+#endif
+
 #ifndef NON_ROOT_FTP
 # ifdef IMPLICIT_TLS
 #  define DEFAULT_FTP_PORT_S "990"
@@ -499,9 +504,9 @@ extern int opt_a, opt_C, opt_d, opt_F, opt_l, opt_R;
 #  define PATH_MAX MAXPATHLEN
 # else
 #  define PATH_MAX 65536U
-Warning: neither PATH_MAX nor MAXPATHLEN were found.
-Remove these lines if you really want to compile the server, but
-the server may be insecure if a wrong value is set here.
+#  error Warning: neither PATH_MAX nor MAXPATHLEN were found.
+#  error Remove these lines if you really want to compile the server, but
+#  error the server may be insecure if a wrong value is set here.
 # endif
 #endif
 #if (PATH_MAX) >= (INT_MAX)
@@ -557,7 +562,7 @@ Your platform has a very large PATH_MAX, we should not trust it.
 
 #ifndef MAX_DATA_SIZE
 # ifdef HAVE_LIBSODIUM
-#  define MAX_DATA_SIZE (40 * 1024 * 1024)
+#  define MAX_DATA_SIZE (70 * 1024 * 1024)
 # elif defined(WITH_LDAP) || defined(WITH_MYSQL) || defined(WITH_PGSQL)
 #  define MAX_DATA_SIZE (16 * 1024 * 1024)       /* Max memory usage - SQL/LDAP need more */
 # else
@@ -620,10 +625,15 @@ Your platform has a very large PATH_MAX, we should not trust it.
 #define WIN32_ANON_DIR "/ftp"
 
 #define STORAGE_PORT(X)  (*storage_port(&(X)))
+#define STORAGE_PORT_CONST(X)  (*storage_port_const(&(X)))
 #define STORAGE_PORT6(X) (*storage_port6(&(X)))
+#define STORAGE_PORT6_CONST(X) (*storage_port6_const(&(X)))
 #define STORAGE_SIN_ADDR(X) (storage_sin_addr(&(X))->s_addr)
+#define STORAGE_SIN_ADDR_CONST(X) (storage_sin_addr_const(&(X))->s_addr)
 #define STORAGE_SIN_ADDR6(X) (storage_sin_addr6(&(X))->s6_addr)
+#define STORAGE_SIN_ADDR6_CONST(X) (storage_sin_addr6_const(&(X))->s6_addr)
 #define STORAGE_SIN_ADDR6_NF(X) (*(storage_sin_addr6(&(X))))
+#define STORAGE_SIN_ADDR6_NF_CONST(X) (*(storage_sin_addr6_const(&(X))))
 
 #ifdef HAVE_SS_LEN
 # define STORAGE_LEN(X) ((X).ss_len)
