@@ -61,9 +61,6 @@ struct reply {
 
 static const char *GETOPT_OPTIONS =
     "0146"
-#ifdef WITH_RFC2640
-    "8:9:"
-#endif
     "Aa:bc:"
 #ifndef NO_STANDALONE
     "BC:"
@@ -104,7 +101,7 @@ static const char *GETOPT_OPTIONS =
     "y:"
 #endif
 #ifdef WITH_TLS
-    "2:Y:J:"
+    "2:3:Y:J:"
 #endif
     "zZ";
 
@@ -114,13 +111,10 @@ static struct option long_options[] = {
     { "logpid", 0, NULL, '1' },
 # ifdef WITH_TLS
     { "certfile", 1, NULL, '2' },
+    { "extcert", 1, NULL, '3' },
 # endif
     { "ipv4only", 0, NULL, '4' },
     { "ipv6only", 0, NULL, '6' },
-# ifdef WITH_RFC2640
-    { "fscharset", 1, NULL, '8' },
-    { "clientcharset", 1, NULL, '9' },
-# endif
     { "chrooteveryone", 0, NULL, 'A' },
     { "trustedgid", 1, NULL, 'a' },
     { "brokenclientscompatibility", 0, NULL, 'b' },
