@@ -13,8 +13,9 @@ int getloadavg(double loadavg[], int nelem)
         nelem--;
         loadavg[nelem] = 0.0;
     } while (nelem != 0);
-    
+
     return nelem;
 }
-
+#else
+extern signed char v6ready;
 #endif
