@@ -8,6 +8,7 @@
 # include "log_mysql.h"
 # include "messages.h"
 # include "crypto.h"
+# include "crypto-sha1.h"
 # include "alt_arc4random.h"
 # include "utils.h"
 
@@ -47,23 +48,28 @@ static char *pw_mysql_escape_string(MYSQL * const id_sql_server,
     size_t from_len;
     size_t to_len;
     char *to;
-    unsigned long tolen;
+    unsigned long escaped_len;
     unsigned int t;
-    unsigned char t1, t2;
+    unsigned char t1, t2, t3, t4;
 
     if (from == NULL) {
         return NULL;
     }
     from_len = strlen(from);
     to_len = from_len * 2U + (size_t) 1U;
-    if ((to = malloc(to_len + (size_t) 2U)) == NULL) {
+    if ((to = malloc(to_len + (size_t) 4U)) == NULL) {
         return NULL;
     }
     t = zrand();
     t1 = t & 0xff;
     t2 = (t >> 8) & 0xff;
+    t = zrand();
+    t3 = t & 0xff;
+    t4 = (t >> 8) & 0xff;
     to[to_len] = (char) t1;
     to[to_len + 1] = (char) t2;
+    to[to_len + 2] = (char) t3;
+    to[to_len + 3] = (char) t4;
     /*
      * I really hate giving a buffer without any size to a 3rd party function.
      * The "to" buffer is allocated on the heap, not on the stack, if
@@ -73,20 +79,17 @@ static char *pw_mysql_escape_string(MYSQL * const id_sql_server,
      * possible instead of doing anything with the heap. We'll end up with
      * a segmentation violation, but without any possible exploit.
      */
-#ifdef HAVE_MYSQL_REAL_ESCAPE_STRING
-    tolen = mysql_real_escape_string(id_sql_server, to, from, from_len);
-#else
-    /* MySQL 3 is obsolete. */
-    tolen = mysql_escape_string(to, from, from_len);
-#endif
-    if (tolen >= to_len ||
+    escaped_len = mysql_real_escape_string(id_sql_server, to, from, from_len);
+    if (escaped_len >= to_len ||
         (unsigned char) to[to_len] != t1 ||
-        (unsigned char) to[to_len + 1] != t2) {
+        (unsigned char) to[to_len + 1] != t2 ||
+        (unsigned char) to[to_len + 2] != t3 ||
+        (unsigned char) to[to_len + 3] != t4) {
         for (;;) {
             *to++ = 0;
         }
     }
-    to[tolen] = 0;
+    to[escaped_len] = 0;
 
     return to;
 }
@@ -328,8 +331,8 @@ void pw_mysql_check(AuthResult * const result,
     char *escaped_peer_ip = NULL;
     char *escaped_decimal_ip = NULL;
     int committed = 1;
-    int crypto_scrypt = 0, crypto_crypt = 0, crypto_mysql = 0, crypto_md5 = 0,
-        crypto_sha1 = 0, crypto_plain = 0;
+    int crypto_argon2 = 0, crypto_scrypt = 0, crypto_crypt = 0,
+        crypto_mysql = 0, crypto_md5 = 0, crypto_sha1 = 0, crypto_plain = 0;
     unsigned long decimal_ip_num = 0UL;
     char decimal_ip[42];
     char hbuf[NI_MAXHOST];
@@ -420,11 +423,14 @@ void pw_mysql_check(AuthResult * const result,
     }
     result->auth_ok--;                  /* -1 */
     if (strcasecmp(crypto, PASSWD_SQL_ANY) == 0) {
+        crypto_argon2++;
         crypto_scrypt++;
         crypto_crypt++;
         crypto_mysql++;
         crypto_md5++;
         crypto_sha1++;
+    } else if (strcasecmp(crypto, PASSWD_SQL_ARGON2) == 0) {
+        crypto_argon2++;
     } else if (strcasecmp(crypto, PASSWD_SQL_SCRYPT) == 0) {
         crypto_scrypt++;
     } else if (strcasecmp(crypto, PASSWD_SQL_CRYPT) == 0) {
@@ -438,7 +444,14 @@ void pw_mysql_check(AuthResult * const result,
     } else {                           /* default to plaintext */
         crypto_plain++;
     }
-#ifdef HAVE_LIBSODIUM
+#ifdef crypto_pwhash_STRPREFIX
+    if (crypto_argon2 != 0) {
+        if (crypto_pwhash_str_verify(spwd, password, strlen(password)) == 0) {
+            goto auth_ok;
+        }
+    }
+#endif
+#ifdef crypto_pwhash_scryptsalsa208sha256_STRPREFIX
     if (crypto_scrypt != 0) {
         if (crypto_pwhash_scryptsalsa208sha256_str_verify
             (spwd, password, strlen(password)) == 0) {
@@ -455,31 +468,25 @@ void pw_mysql_check(AuthResult * const result,
         }
     }
     if (crypto_mysql != 0) {
-#if MYSQL_VERSION_ID < 40100 || defined(USE_OLD_MYSQL_SCRAMBLING)
-        unsigned long hash_res[2];
-        char scrambled_password[MYSQL_CRYPT_LEN];
-
-# if MYSQL_VERSION_ID < 40100
-        hash_password(hash_res, password);
-# else
-        hash_password(hash_res, password, strlen(password));
-# endif
-        snprintf(scrambled_password, sizeof scrambled_password, "%08lx%08lx",
-                 hash_res[0], hash_res[1]);
-#else
         char scrambled_password[42]; /* 2 * 20 (sha1 hash size) + 2 */
+        SHA1_CTX        ctx;
+        unsigned char  h0[20], h1[20];
+        char          *p;
 
-# if MYSQL_VERSION_ID >= 40100 && MYSQL_VERSION_ID < 40101
-        make_scrambled_password(scrambled_password, password, 1, NULL);
-# else
-#  ifdef HAVE_MY_MAKE_SCRAMBLED_PASSWORD
-        my_make_scrambled_password(scrambled_password, password,
-                                   strlen(password));
-#  else
-        make_scrambled_password(scrambled_password, password);
-#  endif
-# endif
-#endif
+        SHA1Init(&ctx);
+        SHA1Update(&ctx, password, strlen(password));
+        SHA1Final(h0, &ctx);
+        SHA1Init(&ctx);
+        SHA1Update(&ctx, h0, sizeof h0);
+        pure_memzero(h0, sizeof h0);
+        SHA1Final(h1, &ctx);
+            *scrambled_password = '*';
+        hexify(scrambled_password + 1U, h1,
+               (sizeof scrambled_password) - 1U, sizeof h1);
+        *(p = scrambled_password) = '*';
+        while (*p++ != 0) {
+            *p = (char) toupper((unsigned char) *p);
+        }
         if (pure_strcmp(scrambled_password, spwd) == 0) {
             goto auth_ok;
         }
