@@ -241,31 +241,6 @@ static int listfile(const PureFileInfo * const fi, const char *name)
     char m[PATH_MAX + 1U];
     const char *format;
 
-#ifndef MINIMAL
-    if (modern_listings != 0) {
-        const char *n;
-        char *alloca_nameline;
-        const size_t sizeof_nameline = PATH_MAX + 256U;
-
-        if (fi == NULL) {
-            n = name;
-        } else {
-            n = FI_NAME(fi);
-        }
-        if ((alloca_nameline = ALLOCA(sizeof_nameline)) == NULL) {
-            return 0;
-        }
-        if ((rval = modernformat(n, alloca_nameline,
-                                 sizeof_nameline, "")) < 0) {
-            ALLOCA_FREE(alloca_nameline);
-            return 0;
-        }
-        addfile(alloca_nameline, suffix);
-        ALLOCA_FREE(alloca_nameline);
-
-        return rval;
-    }
-#endif
     if (fi == NULL) {
         if (lstat(name, &st) < 0) {
             return 0;
@@ -450,7 +425,6 @@ static void outputfiles(int f, void * const tls_fd)
     unsigned int n;
     struct filename *p;
     struct filename *q;
-    char *c_buf; /* buffer with charset of client */
 
     if (!head) {
         return;
@@ -519,14 +493,7 @@ static void outputfiles(int f, void * const tls_fd)
                 pad[1] = '\n';
                 pad[2] = 0;
             }
-#ifdef WITH_RFC2640
-            c_buf = charset_fs2client(q->line);
-            wrstr(f, tls_fd, c_buf);
-            free(c_buf);
-#else
-            (void) c_buf;
             wrstr(f, tls_fd, q->line);
-#endif
             wrstr(f, tls_fd, pad);
             q = q->right;
             free(tmp);
@@ -694,7 +661,6 @@ static void listdir(unsigned int depth, int f, void * const tls_fd,
     char *names;
     PureFileInfo *s;
     PureFileInfo *r;
-    char *c_buf;
     int d;
 
     if (depth >= max_ls_depth || matches >= max_ls_files) {
@@ -713,13 +679,8 @@ static void listdir(unsigned int depth, int f, void * const tls_fd,
             if (FI_NAME(s)[1] == 0 ||
                 (FI_NAME(s)[1] == '.' && FI_NAME(s)[2] == 0)) {
                 listfile(s, NULL);
-#ifndef DONT_LIE_ABOUT_DOT_FILES
-            } else if (dot_read_ok != 0) {
-                d = listfile(s, NULL);
-#else
             } else {
                 d = listfile(s, NULL);
-#endif
             }
         }
         if (!d) {
@@ -741,29 +702,9 @@ static void listdir(unsigned int depth, int f, void * const tls_fd,
                                  name, FI_NAME(r)), sizeof_subdir)) {
                 goto nolist;
             }
-#ifdef WITH_RFC2640
-            c_buf = charset_fs2client(alloca_subdir);
-#else
-            c_buf = alloca_subdir;
-#endif
-#ifndef MINIMAL
-            if (modern_listings == 0) {
-#endif
-#ifdef FANCY_LS_DIRECTORY_HEADERS
-                wrstr(f, tls_fd, "\r\n>----------------[");
-                wrstr(f, tls_fd, c_buf);
-                wrstr(f, tls_fd, "]----------------<\r\n\r\n");
-#else
-                wrstr(f, tls_fd, "\r\n\r\n");
-                wrstr(f, tls_fd, c_buf);
-                wrstr(f, tls_fd, ":\r\n\r\n");
-#endif
-#ifndef MINIMAL
-            }
-#endif
-#ifdef WITH_RFC2640
-            free(c_buf);
-#endif
+            wrstr(f, tls_fd, "\r\n\r\n");
+            wrstr(f, tls_fd, alloca_subdir);
+            wrstr(f, tls_fd, ":\r\n\r\n");
             listdir(depth + 1U, f, tls_fd, alloca_subdir);
             nolist:
             ALLOCA_FREE(alloca_subdir);
@@ -810,23 +751,19 @@ static char *unescape_and_return_next_file(char * const str) {
     return NULL;
 }
 
-void donlist(char *arg, const int on_ctrl_conn, const int opt_l_,
-             const int opt_a_, const int split_args)
+void dolist(char *arg, const int on_ctrl_conn)
 {
     int c;
     void *tls_fd = NULL;
-    char *c_buf;
 
     matches = 0U;
 
-    opt_C = opt_d = opt_F = opt_R = opt_r = opt_t = opt_S = 0;
-    opt_l = opt_l_;
+    opt_a = opt_C = opt_d = opt_F = opt_R = opt_r = opt_t = opt_S = 0;
+    opt_l = 1;
     if (force_ls_a != 0) {
         opt_a = 1;
-    } else {
-        opt_a = opt_a_;
     }
-    if (split_args != 0 && arg != NULL) {
+    if (arg != NULL) {
         while (isspace((unsigned char) *arg)) {
             arg++;
         }
@@ -910,16 +847,9 @@ void donlist(char *arg, const int on_ctrl_conn, const int opt_l_,
             int a;
             char *endarg;
 
-            if (split_args == 0) {
-                endarg = NULL;
-            } else if ((endarg = unescape_and_return_next_file(arg)) != NULL) {
+            if ((endarg = unescape_and_return_next_file(arg)) != NULL) {
                 justone = 0;
             }
-#ifdef DEBUG
-            if (debug != 0) {
-                addreply(226, "Glob: %s", arg);
-            }
-#endif
 
             /* Expand ~ here if needed */
 
@@ -961,23 +891,9 @@ void donlist(char *arg, const int on_ctrl_conn, const int opt_l_,
                     }
                     if (**path != 0) {
                         if (!justone) {
-#ifdef WITH_RFC2640
-                            c_buf = charset_fs2client(*path);
-#else
-                            c_buf = *path;
-#endif
-#ifdef FANCY_LS_DIRECTORY_HEADERS
-                            wrstr(c, tls_fd, "\r\n>-----------------[");
-                            wrstr(c, tls_fd, c_buf);
-                            wrstr(c, tls_fd, "]-----------------<\r\n\r\n");
-#else
                             wrstr(c, tls_fd, "\r\n\r\n");
-                            wrstr(c, tls_fd, c_buf);
+                            wrstr(c, tls_fd, *path);
                             wrstr(c, tls_fd, ":\r\n\r\n");
-#endif
-#ifdef WITH_RFC2640
-                            free(c_buf);
-#endif
                         }
                         if (!chdir(*path)) {
                             listdir(0U, c, tls_fd, *path);
@@ -1041,3 +957,148 @@ end:
         die(421, LOG_ERR, "chdir: %s", strerror(errno));
     }
 }
+
+void donlst(const char *base)
+{
+    char           line[PATH_MAX + 3U];
+    DIR           *dir;
+    void          *tls_fd = NULL;
+    struct dirent *de;
+    size_t         name_len;
+    unsigned int   matches = 0;
+    int            c;
+    int            base_has_trailing_slash = 0;
+
+    if (*base != 0 && chdir(base) != 0) {
+        if (*base++ == '-') {
+            while (!isspace((unsigned char) *base++));
+            while (isspace((unsigned char) *base++));
+            if (*base != 0 && chdir(base) != 0) {
+                addreply_noformat(550, MSG_STAT_FAILURE2);
+                return;
+            }
+        } else {
+            addreply_noformat(550, MSG_STAT_FAILURE2);
+            return;
+        }
+    }
+    if (*base !=0 && base[strlen(base) - 1U] == '/') {
+        base_has_trailing_slash = 1;
+    }
+    if ((dir = opendir(".")) == NULL) {
+        addreply_noformat(550, MSG_STAT_FAILURE2);
+        goto bye;
+    }
+    opendata();
+    if ((c = xferfd) == -1) {
+        goto bye;
+    }
+    doreply();
+#ifdef WITH_TLS
+    if (data_protection_level == CPL_PRIVATE) {
+        tls_init_data_session(xferfd, passive);
+        tls_fd = tls_data_cnx;
+    }
+#endif
+    while ((de = readdir(dir)) != NULL) {
+        if (checkprintable(de->d_name) != 0) {
+            continue;
+        }
+        name_len = strlen(de->d_name);
+        if (name_len > sizeof line - 3U) {
+            continue;
+        }
+        memcpy(line, de->d_name, name_len);
+        line[name_len] = '\r';
+        line[name_len + 1] = '\n';
+        line[name_len + 2] = 0;
+        if (*base) {
+            wrstr(c, tls_fd, base);
+            if (base_has_trailing_slash == 0) {
+                wrstr(c, tls_fd, "/");
+            }
+        }
+        wrstr(c, tls_fd, line);
+        matches++;
+        if (matches >= max_ls_files) {
+            break;
+        }
+    }
+    closedir(dir);
+    wrstr(c, tls_fd, NULL);
+    closedata();
+    if (matches >= max_ls_files) {
+        addreply(226, MSG_LS_TRUNCATED, matches);
+    } else {
+        addreply(226, MSG_LS_SUCCESS, matches);
+    }
+bye:
+    if (chdir(wd) != 0) {
+        die(421, LOG_ERR, "chdir: %s", strerror(errno));
+    }
+}
+
+void domlsd(const char *base)
+{
+    char           line[PATH_MAX + 1];
+    DIR           *dir = NULL;
+    void          *tls_fd = NULL;
+    struct dirent *de;
+    unsigned int   matches = 0;
+    int            c;
+
+    if (*base != 0 && chdir(base) != 0) {
+        if (*base++ == '-') {
+            while (!isspace((unsigned char) *base++));
+            while (isspace((unsigned char) *base++));
+            if (*base != 0 && chdir(base) != 0) {
+                addreply_noformat(550, MSG_STAT_FAILURE2);
+                return;
+            }
+        } else {
+            addreply_noformat(550, MSG_STAT_FAILURE2);
+            return;
+        }
+    }
+    if ((dir = opendir(".")) == NULL) {
+        addreply_noformat(550, MSG_STAT_FAILURE2);
+        goto bye;
+    }
+    opendata();
+    if ((c = xferfd) == -1) {
+        goto bye;
+    }
+    doreply();
+#ifdef WITH_TLS
+    if (data_protection_level == CPL_PRIVATE) {
+        tls_init_data_session(xferfd, passive);
+        tls_fd = tls_data_cnx;
+    }
+#endif
+    while ((de = readdir(dir)) != NULL) {
+        if (checkprintable(de->d_name) != 0 ||
+            modernformat(de->d_name, line, sizeof line, "") < 0) {
+            continue;
+        }
+        wrstr(c, tls_fd, line);
+        wrstr(c, tls_fd, "\r\n");
+        matches++;
+        if (matches >= max_ls_files) {
+            break;
+        }
+    }
+    wrstr(c, tls_fd, NULL);
+    closedata();
+    if (matches >= max_ls_files) {
+        addreply(226, MSG_LS_TRUNCATED, matches);
+    } else {
+        addreply(226, MSG_LS_SUCCESS, matches);
+    }
+bye:
+    if (dir != NULL) {
+        closedir(dir);
+    }
+    if (chdir(wd) != 0) {
+        die(421, LOG_ERR, "chdir: %s", strerror(errno));
+    }
+}
