@@ -5,8 +5,8 @@
 
 int main(void)
 {
-    puts("Please compile the server with --with-extauth\n"
-         "to use this feature. Thank you.");
+    puts("Please compile the server with --with-extauth to use this feature.\n"
+         "Thank you.");
 
     return 0;
 }
@@ -77,7 +77,7 @@ static int init(void)
 #ifndef NON_ROOT_FTP
     if (geteuid() != (uid_t) 0) {
         fprintf(stderr,
-        "Sorry, but you have to be r00t to run this program\n");
+        "Sorry, but you have to be root to run this program\n");
         return -1;
     }
 #endif
@@ -263,6 +263,13 @@ static void callback_client_encrypted(const char *str)
     newenv_str(ENV_AUTHD_ENCRYPTED, str);
 }
 
+static void callback_client_sni_name(const char *str)
+{
+    if (*str != 0) {
+        newenv_str(ENV_AUTHD_CLIENT_SNI_NAME, str);
+    }
+}
+
 static void callback_client_end(const char *str)
 {
     (void) str;
@@ -379,6 +386,11 @@ int listencnx(void)
         perror("Unable to listen the local socket");
         goto bye;
     }
+    if (changeuidgid() < 0) {
+        perror("Identity change");
+        (void) unlink(authd_pid_file);
+        return -1;
+    }
     do {
         if ((clientfd = accept(kindy, NULL, NULL)) == -1) {
             if (exit_authd != 0) {
@@ -454,11 +466,6 @@ int main(int argc, char *argv[])
         dodaemonize();
     }
     updatepidfile();
-    if (changeuidgid() < 0) {
-        perror("Identity change");
-        (void) unlink(authd_pid_file);
-        return -1;
-    }
 #ifdef SIGPIPE
     signal(SIGPIPE, SIG_IGN);
 #endif
