@@ -11,26 +11,7 @@
 # include "globals.h"
 # include "messages.h"
 # include "alt_arc4random.h"
-
-# ifndef DISABLE_SSL_RENEGOTIATION
-#  ifndef SSL3_FLAGS_ALLOW_UNSAFE_LEGACY_RENEGOTIATION
-#   ifndef ACCEPT_SSL_RENEGOTIATION
-#    define DISABLE_SSL_RENEGOTIATION 1
-#   endif
-#  else
-#   ifdef ACCEPT_SSL_RENEGOTIATION
-#    define DISABLE_SSL_RENEGOTIATION 0
-#   endif
-#  endif
-# endif
-
-/*
- * Enabled by default since pure-ftpd 1.0.42, except in broken clients
- * compatibility mode.
- */
-# ifndef ONLY_ACCEPT_REUSED_SSL_SESSIONS
-#  define ONLY_ACCEPT_REUSED_SSL_SESSIONS 1
-# endif
+# include "tls_extcert.h"
 
 static void tls_error(const int line, int err)
 {
@@ -44,8 +25,119 @@ static void tls_error(const int line, int err)
     _EXIT(EXIT_FAILURE);
 }
 
+static int validate_sni_name(const char * const sni_name)
+{
+    static const char *valid_chars =
+        "abcdefghijklmnopqrstuvwxyz.-0123456789_ABCDEFGHIJKLMNOPQRSTUVWXYZ";
+    const char        *pnt = sni_name;
+
+    if (strlen(sni_name) > 255) {
+        return -1;
+    }
+    while (*pnt != 0) {
+        if (strchr(valid_chars, *pnt) == NULL) {
+            return -1;
+        }
+        pnt++;
+    }
+    return 0;
+}
+
+static int tls_create_new_context(const char *cert_file,
+                                  const char *key_file);
+
+static int ssl_servername_cb(SSL *cnx, int *al, void *arg)
+{
+    CertResult  result;
+    const char *sni_name;
+
+    (void) al;
+    (void) arg;
+    if ((sni_name = SSL_get_servername(cnx, TLSEXT_NAMETYPE_host_name))
+        == NULL || *sni_name == 0 || validate_sni_name(sni_name) != 0) {
+        return SSL_TLSEXT_ERR_NOACK;
+    }
+    logfile(LOG_INFO, "SNI: [%s]", sni_name);
+    if (chrooted != 0 || loggedin != 0) {
+        return SSL_TLSEXT_ERR_NOACK;
+    }
+    if (use_extcert == 0) {
+        return SSL_TLSEXT_ERR_OK;
+    }
+    memset(&result, 0, sizeof result);
+    tls_extcert_get(&result, sni_name);
+    if (result.cert_ok != 1) {
+        die(400, LOG_ERR, "Cert handler not ready");
+    }
+    if (result.action == CERT_ACTION_DENY) {
+        die(400, LOG_INFO, MSG_LOGOUT);
+    }
+    if (result.action == CERT_ACTION_DEFAULT) {
+        return SSL_TLSEXT_ERR_OK;
+    }
+    if (result.cert_file == NULL) {
+        if (result.action == CERT_ACTION_STRICT) {
+            die(400, LOG_ERR, "Missing certificate");
+        } else {
+            return SSL_TLSEXT_ERR_OK;
+        }
+    }
+    if (result.key_file == NULL) {
+        result.key_file = result.cert_file;
+    }
+    SSL_CTX_free(tls_ctx);
+    tls_ctx = NULL;
+    if (tls_create_new_context(result.cert_file, result.key_file) != 0) {
+        if (result.action != CERT_ACTION_FALLBACK) {
+            die(400, LOG_ERR, "Invalid certificate");
+        }
+        if (tls_create_new_context(cert_file, key_file) != 0) {
+            die(400, LOG_ERR, "SSL error");
+        }
+    }
+    if ((client_sni_name = strdup(sni_name)) == NULL) {
+        die_mem();
+    }
+    if (tls_cnx != NULL) {
+        const long ctx_options = SSL_CTX_get_options(tls_ctx);
+        SSL_set_SSL_CTX(tls_cnx, tls_ctx);
+# ifdef SSL_CTRL_CLEAR_OPTIONS
+        SSL_clear_options(tls_cnx,
+                          SSL_get_options(tls_cnx) & ~ctx_options);
+# endif
+        SSL_set_options(tls_cnx, ctx_options);
+    }
+    if (tls_data_cnx != NULL) {
+        const long ctx_options = SSL_CTX_get_options(tls_ctx);
+        SSL_set_SSL_CTX(tls_data_cnx, tls_ctx);
+# ifdef SSL_CTRL_CLEAR_OPTIONS
+        SSL_clear_options(tls_data_cnx,
+                          SSL_get_options(tls_cnx) & ~ctx_options);
+# endif
+        SSL_set_options(tls_data_cnx, ctx_options);
+    }
+    return SSL_TLSEXT_ERR_OK;
+}
+
+static void ssl_info_cb(const SSL *cnx, int where, int ret)
+{
+    (void) ret;
+
+    if ((where & SSL_CB_HANDSHAKE_DONE) != 0) {
+        if (cnx == tls_cnx) {
+            tls_cnx_handshook = 1;
+        } else if (cnx == tls_data_cnx) {
+            tls_data_cnx_handshook = 1;
+        }
+    }
+}
+
 static int tls_init_ecdh_curve(void)
 {
+#ifdef SSL_CTRL_SET_ECDH_AUTO
+    SSL_CTX_ctrl(tls_ctx, SSL_CTRL_SET_ECDH_AUTO, 1, NULL);
+    return 0;
+#else
 # ifndef SSL_OP_SINGLE_ECDH_USE
     errno = ENOTSUP;
     return -1;
@@ -71,9 +163,11 @@ static int tls_init_ecdh_curve(void)
 
     return 0;
 # endif
+#endif
 }
 
-static int tls_init_dhparams_default(void)
+#ifndef SSL_CTRL_SET_DH_AUTO
+static int tls_load_dhparams_default(void)
 {
 # ifdef HAVE_DH_GET_2048_256
     DH *dh;
@@ -186,8 +280,9 @@ static const BN_ULONG dh2048_256_q[] = {
 
     return 0;
 }
+#endif
 
-static int tls_init_dhparams(void)
+static int tls_load_dhparams(void)
 {
     BIO *bio;
     DH  *dh;
@@ -208,6 +303,19 @@ static int tls_init_dhparams(void)
     return 0;
 }
 
+static void tls_init_dhparams(void)
+{
+# ifdef SSL_CTRL_SET_DH_AUTO
+    if (tls_load_dhparams() != 0) {
+        SSL_CTX_ctrl(tls_ctx, SSL_CTRL_SET_DH_AUTO, 1, NULL);
+    }
+# else
+    if (tls_load_dhparams() != 0) {
+        tls_load_dhparams_default();
+    }
+# endif
+}
+
 static void tls_init_cache(void)
 {
     static const char *tls_ctx_id = "pure-ftpd";
@@ -219,63 +327,10 @@ static void tls_init_cache(void)
     SSL_CTX_set_timeout(tls_ctx, 60 * 60L);
 }
 
-# ifdef DISABLE_SSL_RENEGOTIATION
-static void ssl_info_cb(const SSL *cnx, int where, int ret)
+static void tls_init_options(void)
 {
-    (void) ret;
+    static int passes;
 
-#  if DISABLE_SSL_RENEGOTIATION == 1
-    if ((where & SSL_CB_HANDSHAKE_START) != 0) {
-        if ((cnx == tls_cnx && tls_cnx_handshook != 0) ||
-            (cnx == tls_data_cnx && tls_data_cnx_handshook != 0)) {
-            die(400, LOG_ERR, "TLS renegociation");
-        }
-        return;
-    }
-#  endif
-    if ((where & SSL_CB_HANDSHAKE_DONE) != 0) {
-        if (cnx == tls_cnx) {
-            tls_cnx_handshook = 1;
-        } else if (cnx == tls_data_cnx) {
-            tls_data_cnx_handshook = 1;
-        }
-#  ifndef SSL_OP_ALLOW_UNSAFE_LEGACY_RENEGOTIATION
-#   if DISABLE_SSL_RENEGOTIATION == 0
-        cnx->s3->flags &= ~(SSL3_FLAGS_NO_RENEGOTIATE_CIPHERS);
-        cnx->s3->flags |= SSL3_FLAGS_ALLOW_UNSAFE_LEGACY_RENEGOTIATION;
-#   else
-        cnx->s3->flags |= SSL3_FLAGS_NO_RENEGOTIATE_CIPHERS;
-#   endif
-#  endif
-        return;
-    }
-}
-# endif
-
-int tls_init_library(void)
-{
-    unsigned int rnd;
-
-    tls_cnx_handshook = 0;
-    tls_data_cnx_handshook = 0;
-# if (OPENSSL_VERSION_NUMBER < 0x10100000L) || !defined(OPENSSL_INIT_LOAD_SSL_STRINGS)
-    SSL_library_init();
-    SSL_load_error_strings();
-    OpenSSL_add_all_algorithms();
-# else
-    OPENSSL_init_ssl(OPENSSL_INIT_LOAD_SSL_STRINGS |
-		     OPENSSL_INIT_LOAD_CRYPTO_STRINGS, NULL);
-    OPENSSL_init_crypto(OPENSSL_INIT_ADD_ALL_CIPHERS |
-			OPENSSL_INIT_ADD_ALL_DIGESTS |
-			OPENSSL_INIT_LOAD_CONFIG, NULL);
-# endif
-    while (RAND_status() == 0) {
-        rnd = zrand();
-        RAND_seed(&rnd, (int) sizeof rnd);
-    }
-    if ((tls_ctx = SSL_CTX_new(SSLv23_server_method())) == NULL) {
-        tls_error(__LINE__, 0);
-    }
 # ifdef SSL_OP_CIPHER_SERVER_PREFERENCE
     SSL_CTX_set_options(tls_ctx, SSL_OP_CIPHER_SERVER_PREFERENCE);
 # endif
@@ -288,10 +343,13 @@ int tls_init_library(void)
     SSL_CTX_set_options(tls_ctx, SSL_OP_NO_TLSv1);
 # endif
 # ifdef SSL_OP_NO_TLSv1_1
-    SSL_CTX_clear_options(tls_ctx, SSL_OP_NO_TLSv1_1);
+    SSL_CTX_set_options(tls_ctx, SSL_OP_NO_TLSv1_1);
 # endif
 # ifdef SSL_OP_NO_TLSv1_2
     SSL_CTX_clear_options(tls_ctx, SSL_OP_NO_TLSv1_2);
+# endif
+# ifdef SSL_OP_NO_TLSv1_3
+    SSL_CTX_clear_options(tls_ctx, SSL_OP_NO_TLSv1_3);
 # endif
     if (tlsciphersuite != NULL) {
         if (SSL_CTX_set_cipher_list(tls_ctx, tlsciphersuite) != 1) {
@@ -299,46 +357,99 @@ int tls_init_library(void)
             _EXIT(EXIT_FAILURE);
         }
     }
+    SSL_CTX_set_info_callback(tls_ctx, ssl_info_cb);
+    if (passes == 0) {
+        SSL_CTX_set_tlsext_servername_callback(tls_ctx, ssl_servername_cb);
+        passes++;
+    }
+    SSL_CTX_set_verify_depth(tls_ctx, MAX_CERTIFICATE_DEPTH);
+}
+
+static void tls_load_cert_file(const char * const cert_file,
+                               const char * const key_file)
+{
     if (SSL_CTX_use_certificate_chain_file(tls_ctx, cert_file) != 1) {
         die(421, LOG_ERR,
             MSG_FILE_DOESNT_EXIST ": [%s]", cert_file);
     }
-    if (SSL_CTX_use_PrivateKey_file(tls_ctx, cert_file,
+    if (SSL_CTX_use_PrivateKey_file(tls_ctx, key_file,
                                     SSL_FILETYPE_PEM) != 1) {
-        tls_error(__LINE__, 0);
+        die(421, LOG_ERR,
+            MSG_FILE_DOESNT_EXIST ": [%s]", key_file);
     }
     if (SSL_CTX_check_private_key(tls_ctx) != 1) {
         tls_error(__LINE__, 0);
     }
+}
+
+static void tls_init_client_cert_verification(const char *cert_file)
+{
+    if (cert_file == NULL) {
+        tls_error(__LINE__, 0);
+    }
+    SSL_CTX_set_verify(tls_ctx, SSL_VERIFY_FAIL_IF_NO_PEER_CERT |
+                       SSL_VERIFY_PEER, NULL);
+    if (SSL_CTX_load_verify_locations(tls_ctx, cert_file, NULL) != 1) {
+        tls_error(__LINE__, 0);
+    }
+}
+
+static int tls_create_new_context(const char *cert_file,
+                                  const char *key_file)
+{
+# ifdef HAVE_TLS_SERVER_METHOD
+    if ((tls_ctx = SSL_CTX_new(TLS_server_method())) == NULL) {
+        tls_error(__LINE__, 0);
+    }
+# else
+    if ((tls_ctx = SSL_CTX_new(SSLv23_server_method())) == NULL) {
+        tls_error(__LINE__, 0);
+    }
+# endif
+    tls_init_options();
     tls_init_cache();
-# ifdef SSL_CTRL_SET_ECDH_AUTO
-    SSL_CTX_ctrl(tls_ctx, SSL_CTRL_SET_ECDH_AUTO, 1, NULL);
-# else
-    tls_init_ecdh_curve();
-# endif
-# ifdef SSL_CTRL_SET_DH_AUTO
-    if (tls_init_dhparams() != 0) {
-        SSL_CTX_ctrl(tls_ctx, SSL_CTRL_SET_DH_AUTO, 1, NULL);
-    }
-# else
-    if (tls_init_dhparams() != 0) {
-        tls_init_dhparams_default();
-    }
-# endif
-# ifdef DISABLE_SSL_RENEGOTIATION
-#  if DISABLE_SSL_RENEGOTIATION == 0 && defined(SSL_OP_ALLOW_UNSAFE_LEGACY_RENEGOTIATION)
-    SSL_CTX_set_options(tls_ctx, SSL_OP_ALLOW_UNSAFE_LEGACY_RENEGOTIATION);
-#  endif
-    SSL_CTX_set_info_callback(tls_ctx, ssl_info_cb);
-# endif
-    SSL_CTX_set_verify_depth(tls_ctx, 6);
+    tls_load_cert_file(cert_file, key_file);
     if (ssl_verify_client_cert) {
-        SSL_CTX_set_verify(tls_ctx, SSL_VERIFY_FAIL_IF_NO_PEER_CERT |
-                           SSL_VERIFY_PEER, NULL);
-        if (SSL_CTX_load_verify_locations(tls_ctx, cert_file, NULL) != 1) {
-            tls_error(__LINE__, 0);
-        }
+        tls_init_client_cert_verification(cert_file);
     }
+    tls_init_ecdh_curve();
+    tls_init_dhparams();
+
+    return 0;
+}
+
+static void tls_init_rnd(void)
+{
+    unsigned int rnd;
+
+    while (RAND_status() == 0) {
+        rnd = zrand();
+        RAND_seed(&rnd, (int) sizeof rnd);
+    }
+}
+
+int tls_init_library(void)
+{
+    tls_cnx = NULL;
+    tls_data_cnx = NULL;
+    tls_ctx = NULL;
+
+# if (OPENSSL_VERSION_NUMBER < 0x10100000L) || !defined(OPENSSL_INIT_LOAD_SSL_STRINGS)
+    SSL_library_init();
+    SSL_load_error_strings();
+    OpenSSL_add_all_algorithms();
+# else
+    OPENSSL_init_ssl(OPENSSL_INIT_LOAD_SSL_STRINGS |
+                     OPENSSL_INIT_LOAD_CRYPTO_STRINGS, NULL);
+    OPENSSL_init_crypto(OPENSSL_INIT_ADD_ALL_CIPHERS |
+                        OPENSSL_INIT_ADD_ALL_DIGESTS |
+                        OPENSSL_INIT_LOAD_CONFIG, NULL);
+# endif
+    tls_init_rnd();
+    tls_create_new_context(cert_file, key_file);
+    tls_cnx_handshook = 0;
+    tls_data_cnx_handshook = 0;
+
     return 0;
 }
 
@@ -431,11 +542,6 @@ int tls_init_data_session(const int fd, const int passive)
         }
         break;
     }
-# if ONLY_ACCEPT_REUSED_SSL_SESSIONS
-    if (broken_client_compat == 0 && SSL_session_reused(tls_data_cnx) == 0) {
-        tls_error(__LINE__, 0);
-    }
-# endif
     if ((cipher = SSL_get_current_cipher(tls_data_cnx)) != NULL) {
         int strength_bits = SSL_CIPHER_get_bits(cipher, NULL);
 
@@ -451,6 +557,7 @@ int tls_init_data_session(const int fd, const int passive)
 void tls_close_session(SSL ** const cnx)
 {
     unsigned int retries = 10U;
+    unsigned int max_shutdowns = 2;
 
     if (*cnx == NULL) {
         return;
@@ -458,9 +565,10 @@ void tls_close_session(SSL ** const cnx)
 retry:
     switch (SSL_shutdown(*cnx)) {
     case 0:
-        SSL_shutdown(*cnx);
-    case SSL_SENT_SHUTDOWN:
-    case SSL_RECEIVED_SHUTDOWN:
+        if (--max_shutdowns > 0) {
+            goto retry;
+        }
+    case 1:
         break;
     default: {
         switch (SSL_get_error(*cnx, -1)) {
